'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/fill/directory.py $"
__revision__ = "$Id: directory.py 971 2008-03-18 19:49:42Z mario $"

from sets import Set
import qp.fill.directory
from gz.pub.common import get_publisher, get_request, get_user
from gz.pub.common import get_qcomp, get_qcomps, get_effective_qcomp
from gz.pub.common import is_last_traversed, is_requested_qcomp
from gz.pub.common import script_name_path
from gz.lib.design_patterns import bunch


class Directory(qp.fill.directory.Directory):

    def _q_traverse (self, path):
        ''' (path: [string]) -> None | Stream | object
        Remember the (dir, component) being traversed.
        '''
        # [un]common.set_qcomp(self, path[0])
        info = get_publisher().get_hit().get_info()
        if not info.has_key('qcomps'):
            info['qcomps'] = []
        info['qcomps'].append((self, path[0]))
        info['done_traversal'] = len(path)==1
        return qp.fill.directory.Directory._q_traverse(self, path)

    def get_exports(self):
        ''' () -> [ (component, attribute, crumb, title) ]
        Generator of qp-compliant 4-tuple exports, that are allowed 
        for the current user context.
        '''
        for x in self.get_allowed_exportables():
            yield x.to_4tuple()

    exportables = []
    def get_exportables(self):
        ''' () -> [exportable]
        Generator of ALL eXportables for this directory. 
        Subclasses should override -- an efficient way (naturally cached) is 
        to simply redefine self.exportables. Otherwise, one may need to 
        provide own caching mechanism for the generated eXportable instances.
        Performance is an issue as this may be called several times per hit.
        '''
        for x in self.exportables:
            yield x

    def get_x(self):
        ''' () -> exportable | None
        Retrieve the current request's exportable object --  
        current directory must have incoming component defined. 
        Note: named get_x() and not get_exportable() for friendlier grepping.
        '''
        directory, component = get_qcomp()
        if directory is not self:
            print 'WARN: Directory.get_x() - expected directory', 
            print '(%s) but got (%s)' % (self, directory)
            return None
        return self.get_x_comp(component)

    def get_x_comp(self, component):
        ''' (component:str) -> exportable | None
        Retrieve this dir's exportable object for component.
        '''
        for x in self.get_exportables():
            if x.component == component:
                return x

    def get_path(self, qcomps=None):
        ''' () -> str | None
        Return the URL path to this *directory* (without the child component,
        if any) where: 
        - the path is that "implied" by a supplied qcomps parameter
        - or is the currently traversed path.
        On lock_responding_now responses, return None.
        Note that this is different from request.get_path().
        '''
        comps = []
        if qcomps is None:
            qcomps = get_qcomps()
        for qcomp in qcomps:
            if self is qcomp[0]: 
                # Ensure path is to *this* directory, not any of its exports.
                # The url_path to a *dir* always ends in a slash. 
                comps.append('')
                return script_name_path('/'.join(comps))
            comps.append(qcomp[1])
        # self is not a traversed dir in qcomps... must be respond_now_qcomp()
        return None

    def get_menu(self, root_path=None):
        ''' (root_path:str) [ (root_path, component, crumb, title, directory) ]
        Return a menu of allowed exports for this directory, in crumb format.
        root_path must be specified if this directory is not on traversal path.
        '''
        menu = []
        if root_path is None: 
            root_path = self.get_path() 
        if root_path is None: 
            # Self must be transferred to after a lock_responding_now()
            if self is not get_effective_qcomp()[0]: 
                print 'WARN: Directory.get_menu(None) - failed to get a',
                print 'non-None root_path (%s)' % (self)
            # OK, but we probably want menu of the real last traversed dir (?)
            subbed_dir = get_qcomp()[0]
            return subbed_dir.get_menu()
        for x in self.get_allowed_exportables(homing=1):
            if x.crumb:
                path = x.component
                if not x.is_external():
                    path = root_path + path
                    if x.slashed:
                        path = path + '/'
                menu.append((path, x.component, x.crumb, x.title, self))
        return menu

    def get_crumbs(self, root=False, qcomps=None):
        ''' (root:bool, qcomps:[(dir, comp)]) ->
                            [ (root_path, component, crumb, title, directory) ]
        Returns the list of crumbs for:
        - either the "implied path" from a supplied qcomps parameter
        - or else ASSUMES the currently traversed path
        When root is True, attempt a first crumb for site root "Home". 
        Note: crumbs[-1] -> crumb for this request. 

        crumb:tuple(
            root_path : str, full path from root, & trailing slash if needed
            component : str, the component for this eXportable
            crumb : str|None, the crumb label
            title : str|None
            directory : the directory of this eXportable
        )
        '''
        crumbs = []
        if qcomps is None:
            qcomps = get_qcomps()
        # For each qcomp, need next root_path
        dir, comp = qcomps[0]
        root_path = dir.get_path(qcomps=qcomps) 
        for next_dir, next_comp in qcomps[1:]:
            x = dir.get_x_comp(comp)
            next_root_path = next_dir.get_path(qcomps=qcomps) 
            if x is not None:
                crumbs.append((next_root_path, comp, x.crumb, x.title, dir))
            root_path, comp, dir = next_root_path, next_comp, next_dir
        # Process last comp, if not ''
        if comp:
            x = dir.get_x_comp(comp)
            if x is not None:
                crumbs.append((root_path+comp, comp, x.crumb, x.title, dir))
        if root:
            dir = get_publisher().get_root_directory()
            if not (crumbs and crumbs[0][4]==dir and crumbs[0][1]==''):
                x = dir.get_x_comp('')
                if x is not None:
                    if dir.is_allowed(allow=x.allow, homing=1):
                        crumbs.insert(0, 
                            (dir.get_path(), '', x.crumb, x.title, dir))
        return crumbs

    def get_crumb_menus(self, offset=0, offset_to=None):
        ''' (offset:int, offset_to:either(int, None)) -> [
                [(root_path, component, crumb, title, directory)] 
                                    ]
        Returns a list of menu lists, of allowed exports in crumb format, 
        one for each directory on the currently traversed path.
        ASSUMPTION : self is last traversed directory.
        NOTE: Conceptually equivalent to QP's get_crumb_tree().'
        '''
        dirs = Set()
        crumb_menus = []
        for root_path, comp, crumb, title, dir in self.get_crumbs():
            dirs.add(dir)
            crumb_menus.append(dir.get_menu())
        root_path = self.get_path()
        if root_path is not None and root_path.endswith('/'): 
            # This (requested) export is a node, so may have children.
            # But, the last dir traversed is not necessarily same as one for 
            # last component on root_path, so check and add if not.
            # On lock_responding_now responses, self.get_path() is None.
            last_dir_traversed = get_qcomp()[0]
            if not last_dir_traversed in dirs:
                crumb_menus.append(last_dir_traversed.get_menu()) 
        if offset>0 or offset_to is not None:
            return crumb_menus[ offset : offset_to ]
        return crumb_menus

    def get_crumbed_menu(self, offset=0):
        ''' (offset:int) -> [ either(
                        (root_path,component,crumb,title,directory),
                        [(root_path,component,crumb,title,directory)] )]
        Re-organizes the crumb_menus, by replacing each crumb (for which one
        of the other menus is the sub-menu) by the sub-menu itself. In this
        structure, the first element of each list is the "index" menu item 
        for that node directory.
        ASSUMPTION : self is last traversed directory.
        '''
        crumb_menus = self.get_crumb_menus(offset=offset)
        crumb_menus.reverse()
        for i, menu in enumerate(crumb_menus[:-1]):
            for j, crumb in enumerate(menu):
                if type(crumb) == list:
                    continue # already spliced
                component = crumb[0].rstrip('/')
                upmenu = crumb_menus[i+1]
                spliced_up = False
                for k, upcrumb in enumerate(upmenu):
                    upcomponent = upcrumb[0].rstrip('/')
                    if component == upcomponent:
                        upmenu[k] = menu
                        spliced_up = True
                        break
                if not spliced_up:
                    # try append at end of upmenu
                    upmenu.append(menu)
                break
        if len(crumb_menus) and len(crumb_menus[-1]):
            if type(crumb_menus[-1][0]) == list:
                return crumb_menus[-1][0]
            else:
                return crumb_menus[-1]
        else:
            return crumb_menus
    
    def get_allowed_exportables(self, exportables=None, homing=None):
        ''' (   exportables : either(None, [X]), 
                homing      : either(None, 1, 2, 3) ) -> [X]

        The contextual homing level requirement is processed as follows: 
            None : unknown, needs to be set 
            1 : indirect (e.g. menu) 
            2 : is_last_traversed
            3 : is_requested_qcomp 
        '''
        if exportables is None: 
            exportables = self.get_exportables()
        user = get_user()
        for x in exportables:
            if not x.allow: 
                # no access checking needed
                yield x
                continue
            _homing = homing
            if _homing is None: 
                if is_requested_qcomp(self, x.component):
                    _homing = 3
                elif is_last_traversed(self, x.component):
                    _homing = 2
                else:
                    _homing = 1
            if self.is_allowed(x.allow, user, _homing):
                yield x
                continue
    
    def is_allowed(self, allow=None, user=None, homing=1):
        ''' (   allow   : either(None, bool, sequence(str)), 
                user    : either(None, instance(User)), 
                homing  : either(1, 2, 3) 
            ) -> bool

        Evaluate the allow tuple, taking into account whether this object (the 
        granter of this *allow*) is being requested (by user, the grantee) for 
        (a) specific "direct" access to itself or 
        (b) "indirect" access, only for information about itself e.g. due to 
        its association to another object, such as whether this object should 
        be included in a requested directory's menu. 
        
        This function knows nothing about exports/exportables, and therefore 
        can be called from any application context (such as from within 
        _q_lookup() and thus with no explicit X instance) or on behalf 
        of any object. 
        
        The possibility to specify user is provided for 2 reasons:  
        1) To be able to determine rights on behalf of another user 
           as well as for a resource other than the current request. 
        2) As an optimization, to reduce unnecessary repetitive get calls 
           (typically this is called within loops over all exportables).

        The contextual homing level requirement is processed as follows
        (NOTE that here it may not be None) : 
            1 : indirect (menu) 
            2 : is_last_traversed
            3 : is_requested_qcomp 
        '''
        if allow is None or allow is True:
            return True
        if allow is False:
            return False
        
        if user is None:
            user = get_user()
        
        # anonymous & administrator are the extreme ends of the spectrum --
        # they can "shortcut", in opposite ways, all other permissions
        if 'anonymous' in allow:
            if not user or (homing>1): #+ should always return "unmenu-ed" True?
                return True 
        if user.is_admin(): 
            if not 'anonymous' in allow: #+ (if above change), no need for this
                return True 
        
        publisher = get_publisher()
        
        # flag for postponing any lock_responding_now calls to end
        unauthorized = False
        
        # all role-permissions on an exportable are always OR-combined
        # (as opposed to role-permissions on different exportables, 
        # e.g. parent and child, that are always AND-combined). 
        # 
        # from within for loop, may only safely return True 
        for rp in allow:
            if rp == 'authenticated':
                # If 'authenticated' is required, resources should not require
                # other "more permissive" restrictions, specifically 'anonymous'. 
                # I.e. no need to postpone this lock_responding_now check.
                if (homing>1): #+ comment why needed
                    publisher.ensure_signed_in()
                return True
            elif rp == 'callback':
                # Callbacks are likely _q_lookup'ed, no explicit X.
                if (homing>1): #+ 
                    # Allow only if request's & referer's server are the
                    # same. Note that the Referer is easy to fake.
                    request = get_request()
                    if not request.environ.get('HTTP_REFERER',''
                            ).startswith('%s://%s'%(
                            request.get_scheme(), request.get_server())):
                        unauthorized = True
                else:
                    return True
            elif rp == 'administrator':
                if (homing>1): #+ comment why needed
                    unauthorized = True
            elif user.is_granted(rp, True):
                return True
            elif user.is_granted(rp+'_traverse', True):
                if (homing<3): #+ comment why needed
                    return True
            
        if user.is_granted('non-administrator', True):
            # non-administrator is never set on resources, but is 
            # implicitly granted by all objects that do not require 
            # admininistrator. It must be explicitly granted to users. 
            if not ('administrator' in allow or 'anonymous' in allow):
                return True
        
        # postponed lock_responding_now calls
        if unauthorized:
            publisher.unauthorized()
        
        if (homing>1): #+ comment why needed
            publisher.unauthorized()
        return False
    
