'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/fill/register_dir.py $"
__revision__ = "$Id: register_dir.py 964 2008-03-17 14:03:29Z mario $"

from qp.mail.send import sendmail
from gz.pub.common import get_publisher, get_request, get_dhi, page
from gz.fill.directory import Directory
from gz.fill.exportable import X
from gz.gizmo.gform import GForm
from gz.gizmo.gspec import Spec
from gz.gizmo.gspec import pattern, equalwidget, shadowwidget, both, length


class RegisterDir(Directory):

    exportables = [
        X('', 'register', 'Register', 'Please Register', ('anonymous',)),
    ]
    def _q_lookup(self, component):
        if component.endswith('.json'):
            # Example of access controlling _q_lookups
            if self.is_allowed(allow=('anonymous',), homing=2):
                field_name = component[:-5]
                get_publisher().json_callback_fs(self, field_name)

    def define_form(self, gizmoid, dhi):
        form = GForm(gizmoid, dhi=dhi, use_tokens=True, check_evt='onchange')
        form.add_string('first', title="First Name")
        form.add_string('last', title="Last Name") 
        form.add_string('userid', title="Username", 
            required=True, 
            hint=('Only alphanumeric _- and cannot start with a number. '
                  'Lowercase (case-insensitive).'),
            spec=Spec(pattern('^[a-zA-Z_][a-zA-Z0-9_]*$'),
                'Userid is malformed.', 'Userid is correctly formed.'),
            callback='json',
            onblur='javascript:this.value=this.value.toLowerCase();')
        form.add_string('email', title="Email", 
            required=True,
            hint='Valid email address, and not used by another user.',
            spec=Spec(pattern('^.+@.+\..{2,4}$'),
                'Email is malformed.', 'Email is correctly formed.'),
            onblur='javascript:this.value=this.value.toLowerCase();')
        form.add_password('passwd1', title="Password", 
            required=True,
            hint="Case sensitive. At least 4 chars long. At least 1 digit.",
            spec=shadowwidget(form, 'passwd2'))
        form.add_password('passwd2', title="Password Confirm", 
            required=True, 
            hint="Must be same as first password.",
            spec=Spec(
                    both(
                        Spec(equalwidget(form, 'passwd1'), 'not equal.'),
                        Spec(pattern('^\d|^\D+\d'), 'at least 1 digit.'),
                        Spec(length(4, None), 'too short.')),
                    'Password is not OK... ', 'Password is OK.'))
        form.add_submit('register', value='Register', check_evt=False)
        return form
        
    def register(self):
        publisher, request = get_publisher(), get_request()
        dhi = get_dhi() # retrieve/init dhi
        form = self.define_form('reg', dhi)
        
        # gform.get_required_names()
        required_field_names = [ widget.name 
            for widget in form.get_all_widgets() if widget.required ]
        
        if form.is_submitted() and not form.has_errors():
            #sendmail('register test', repr(form), 
            #   [str(request.get_field('email'))])
            pass # ok, proceed ...

        return self.register_page(dhi.x.title, form, template='register')

    def register_page(self, title, form, **kw):
        return page(title, form.render(), **kw)

    # -- field checkers ------------------------------------------------------
    '''
    Field callback checker methods define validation logic that requires 
    server-side resourses on each call. They receive a field_status object,
    that they must update as necessary. This way they also serve a double 
    purpose:
    a) As the actual AJAX callbacks, being called by the publisher with 
       an initialized field_status object.
    b) They are systematically called during the widget's server-side 
       parsing and validation -- there is no reliance on client-side
       javascript for field validation. 
       
    Field callback functions expect an initialized field_status object:
        fs = {
            name : str,        # field name
            value : anything,  # field value
            ok : bool,         # value is ok
            message : str,     # error or ok message
        }

    Naming convention for field callback methods:
        'callback_fs_%(field_name)s'
        
    To declare:
    - provide a _q_lookup that intercepts requests for component='<name>.json'
      and dispatches them to publisher.json_callback_fs(dir, '<name>').
    - On same directory that supports the form, define a method 
      'callback_fs_<name>', that accepts an initialized fs object, and 
      updates it accordingly. 
    '''
    _userid_attr_name = 'id'
    def callback_fs_userid(self, fs):
        publisher, value = get_publisher(), fs['value']
        User = publisher.get_user_class()
        user = publisher.get_users().get(value)
        if not publisher.match(value, User, self._userid_attr_name):
            fs['ok'] = False
            fs['message'] = 'Sorry, need a valid user name.'
        elif user:
            fs['ok'] = False
            fs['message'] = 'Sorry, name is taken.'
        else:
            fs['message'] = 'OK, name is available.'
        return fs
