'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/gizmo/gizmo.py $"
__revision__ = "$Id: gizmo.py 975 2008-03-19 10:44:16Z mario $"

from qpy import h8
from gz.fill.dhi import Dhi
from gz.pub.common import get_dhi
from gz.lib.evaluator import Evaluator

class Gizmo(object):
    ''' Abstract Gizmo class 
    '''

    # To be able to subclass server-side, with no change to client-side
    # If None, client-side class name = self.__class__.__name__
    js_gizmo_name = None 

    # css/js file dependencies
    css_files = []
    js_files = [
        '/gza/MochiKit/MochiKit.js', 
        '/gza/gizmo.js'
    ]
    js_init = "var %(gizmoid)s = new %(js_gizmo_name)s('%(gizmoid)s');"
    
    ''' Exposed element names : 
    - A page-unique id is implied for each name, see self.get_id(name). 
      To output the actual page-unique id value for the name, template 
      boilerplate code for Gizmo objects should use '%(get_id('name'))s'. 
    - Elements are accessed in javascript on the client with 
      gizmo.get_elem(name). 
    '''
    elem_names = []
    # Note: to bypass an IE problem when a js var name is equal to a dom id, 
    # we adopt the convention that the id of an enclosing DOM element for 
    # a gizmo is: gizmoid_gizmoid

    def __init__(self, 
            gizmoid,                # str : required for all gizmos 
                                    # must be a valid javascript variable name
                                    
            # Base set of optional init parameters
            dhi=None,               # instance(Dhi) : sets hit's dhi if None
            css_codes=None,         # [h8] : CSS code boilerplates
            onload_actions=None,    # [h8] : JS onload action boilerplates
            js_codes=None,          # [h8] : JS code boilerplates
        ):
        self.dhi = dhi or get_dhi()
        self.register(gizmoid)
        self.css_codes = css_codes or [] 
        self.onload_actions = onload_actions or [] 
        self.js_codes = js_codes or []
        if self.js_init not in self.js_codes:
            self.js_codes.append(self.js_init)

        # Global vars to be passed to gizmo's Evaluator for rendering.
        # Access to gvar attrs should be through the methods:
        # Gizmo.get(name), Gizmo.set(name, value).
        gvars = self.gvars = {}
        self.eval = Evaluator(gvars)
        gvars['gizmoid'] = gizmoid
        gvars['js_gizmo_name'] = (
                            self.js_gizmo_name or self.__class__.__name__)
        gvars['get_id'] = self.get_id
        
        # Instance-specific CSS styles
        self.styles = {}
        self.init_styles()
        
    # ------------------------------------------------------------------
    
    def register(self, gizmoid):
        assert self.dhi.gizmos.get(gizmoid) is None
        self.dhi.gizmos[gizmoid] = self

    def get(self, name, default=None):
        return self.gvars.get(name, default)
    
    def set(self, name, val):
        self.gvars[name] = val

    def add_to(self, name, val): 
        ''' (name:str, val:any) -> None 
        Appends (creates if None) to the gvar "name" list attribute.
        '''
        add_to_list = self.gvars.get(name)
        if add_to_list is None:
            add_to_list = self.gvars.setdefault(name, [])
        add_to_list.append(val)
        
    def get_id(self, name):
        return '%s_%s' % (self.gvars['gizmoid'], name)
    
    def set_style(self, css_class, prop, val):
        if not self.styles.has_key(css_class):
            self.styles[css_class] = {}
        self.styles[css_class][prop] = val 

    def add_css_codes(self, *css_codes):
        for css_code in css_codes:
            if css_code not in self.css_codes:
                self.css_codes.append(css_code)

    def add_js_codes(self, *js_codes):
        for js_code in js_codes:
            if js_code not in self.js_codes:
                self.js_codes.append(js_code)

    def add_onload_actions(self, *onload_actions):
        for onload_action in onload_actions:
            if onload_action not in self.onload_actions:
                self.onload_actions.append(onload_action)

    def init_styles(self):
        # Calls to self.set_style(gizmo_exposed_name, css_property, value). 
        # The instance-specific CSS will be inlined in the generated page HEAD.
        pass
        
    # ------------------------------------------------------------------
    
    def prepare_dhi(self, **kw):
        ''' "Upload" dhtml info for this Gizmo instance to the current hit's
        Dhi instance, checking for unncecessary duplication. This method is 
        called at render time, and should be called once only. 
        
        Gizmo render_* methods support arbitrary **kw args, that are set as 
        Gizmo.eval.locals. Small optimization: we do not need to pass down 
        **kw here, as Gizmo.eval.locals, after pre-set, will not change.
        '''
        self.eval.set_locals(kw)
        dhi = self.dhi
        if self.css_files: 
            dhi.add_css_files(*self.css_files)
        if self.js_files: 
            dhi.add_js_files(*self.js_files)
        if self.css_codes: 
            dhi.add_css_codes(*self.render_css())
        if self.styles:
            dhi.add_css_codes(*self.render_styles())
        if self.js_codes: 
            dhi.add_js_codes(*self.render_js())
        if self.onload_actions: 
            dhi.add_onload_actions(*self.render_onload_actions())

    # -- renderers -----------------------------------------------------

    # generic rendering function
    def render_bp(self, boilerplate, **kw):
        # (boilerplate:str) -> h8
        if kw:
            self.eval.set_locals(kw)
        return h8(boilerplate) % (self.eval)

    def render_onload_actions(self, **kw):
        # () -> [h8]
        if kw:
            self.eval.set_locals(kw)
        return [ h8(onload_action) % (self.eval) 
                 for onload_action in self.onload_actions ] 

    def render_styles(self, **kw):
        # () -> [h8]
        if kw:
            self.eval.set_locals(kw)
        return [ h8('#%s_%s {\n%s}\n') % (
                    self.gvars['gizmoid'], css_class, h8.from_list([ 
                        '  %s: %s;\n' %(property, value) 
                        for property, value in properties.iteritems() ]))
                for css_class, properties in self.styles.iteritems() ]

    def render_css(self, **kw):
        # () -> [h8]
        if kw:
            self.eval.set_locals(kw)
        return [ h8(css_code) % (self.eval) 
                 for css_code in self.css_codes ]
        
    def render_js(self, **kw):
        # () -> [h8]
        if kw:
            self.eval.set_locals(kw)
        return [ h8(js_code) % (self.eval) 
                 for js_code in self.js_codes ]

    def render(self, **kw):
        # () -> h8 
        self.prepare_dhi(**kw) 
        raise NotImplementedError
