'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/gizmo/gspec.py $"
__revision__ = "$Id: gspec.py 975 2008-03-19 10:44:16Z mario $"

from qp.lib.spec import Specification, SpecOperator, match, equal
from qp.lib.spec import pattern, length
from gz.pub.common import get_request

# ----------------------------------------------------------------------------
# various specs, may be a combination of any of the following : 
# 
# a) any literal value, e.g. None, 1, 'abc', <instance X>. A "Typogram".
# b) any type, e.g. int, str, <class X>. A "Typogram".
# c) any instance of a sub-class of qp.lib.spec.SpecOperator, including in 
#    particular ConnectiveSpecOperator
# d) any instance of the free-floating Anything class

class equalwidget(equal):
    def __init__(self, form, widget_name):
        self.args = (form, widget_name) 
        self.value = get_request().get_field(widget_name)

class shadowwidget(SpecOperator):
    def __init__(self, form, widget_name):
        self.args = (form, widget_name) 
        self.widget_name = widget_name
        self.form = form
    def __call__(self, value):
        return self.form.get_widget(self.widget_name).spec(value)

# ----------------------------------------------------------------------------

class Spec(Specification):
    ''' Wrapper class for QP or Gizmo Typograms.
    Inherits from qp.lib.spec.Specification (that adds doc attr plus 
        some other methods).
    Adds messages for ok/error.
    Adds a render_js() method.
    
    Usage: 
    from gz.gizmo.gspec import Spec
    '''
    
    def __init__(self, spec, mer=None, mok=None, doc=''):
        Specification.__init__(self, spec, doc)
        self.mer = mer # message_error : str
        self.mok = mok # message_ok : str
        
    def render_js(self):
        # () -> str
            
        s = self.spec
            
        if isinstance(s, pattern):
            return self.js_new_Spec('Pattern', '/%s/'%(s.args[0]))
            
        elif isinstance(s, both):
            js_specs = [];
            for arg in s.args:
                js_specs.append(ensure_Spec(arg).render_js())
            return self.js_new_Spec('Both', '[%s]' % (', '.join(js_specs)))
        
        elif isinstance(s, either):
            js_specs = [];
            for arg in s.args:
                js_specs.append(ensure_Spec(arg).render_js())
            return self.js_new_Spec('Either', '[%s]' % (', '.join(js_specs)))

        elif isinstance(s, equalwidget):
            return self.js_new_Spec('EqualWidget', "'%s'"%(s.args[1]))
            
        elif isinstance(s, shadowwidget):
            return self.js_new_Spec('ShadowWidget', "'%s'"%(s.args[1]))
            
        elif isinstance(s, length):
            return self.js_new_Spec('Length', '%s, %s' % (s.min, s.max or 'null'))

        else:
            print 'Spec (%s) is unsupported on client' % (s)
            return ''

    # Support for render_js()
    
    def js_new_Spec(self, js_type, js_spec_params):
        return "new %s(%s, %s, %s)" % (
                    js_type, 
                    js_spec_params, 
                    js_str_param(self.mer), 
                    js_str_param(self.mok))

# Support for Spec.render_js()

def js_str_param(message):
    if message is None:
        return 'null'
    return "'%s'"%(message)

def ensure_Spec(arg, mer=None, mok=None, doc=None):
    # "casts" a spec to an instance of Spec()
    if not isinstance(arg, Spec):
        # use default mer, mok
        if not isinstance(arg, Specification):
            # a Typogram or SpecOperator or Anything
            return Spec(arg, mer, mok, doc) 
        else: 
            return Spec(arg.spec, mer, mok, doc=arg.doc or doc)
    return arg

# ----------------------------------------------------------------------------
# ConnectiveSpecOperators 
# Different than QP's, to be able to know which of the constituent specs 
# caused the match to fail or to succeed (break_spec). This allows the 
# possibility to select a more appropriate error message. 

class ConnectiveSpecOperator(SpecOperator):
    
    def __init__(self, *specs):
        self.specs = specs
        self.args = self.specs
        self.break_spec = None
        
    def __call__(self, value):
        self.break_spec = None
        for spec in self.specs:
            is_conclusive = self.check_one(value, spec)
            if is_conclusive is not None:
                self.break_spec = spec
                return is_conclusive
        return True
    
    def check_one(self, value, typogram):
        ''' 
        if, from this unitary test, we can already conclude that the 
        ConnectiveSpecOperator has:
            succeeded, then return True 
            failed, then return False
        If no conclusion can be deduced yet:
            return None
        '''
        raise NotImplementedError

class no(ConnectiveSpecOperator):
    '''
    Use this to specify a negation of specs.
    Example: no(None)
    '''
    
    def check_one(self, value, spec):
        if match(value, spec):
            return False
        return None

class both(ConnectiveSpecOperator):
    '''
    Use this to specify a conjunction of specs.
    Example:
      both([int], length(3)) specifies values that are lists of 3 ints.
    '''
    
    def check_one(self, value, spec):
        if not match(value, spec):
            return False
        return None

    def explain_difference(self, value):
        error = format_expected_got(value, self)
        for spec in self.specs:
            if not match(value, spec):
                error += "  (which does not match %s)" % format_spec(spec)
                break
        return error

class either(ConnectiveSpecOperator):
    '''
    Use this to specify a disjunction of specs.
    Examples:
      either(int, str) specifies values that are ints or strs.
    '''
    
    def check_one(self, value, spec):
        if match(value, spec):
            return True
        return None

class proper(both):
    '''
    Use this to specify an instance with attributes that agree with the
    attribute specifications for this class.
    If arguments are provided, they are additional specs which the instance
    must satisfy.
    '''

    def __call__(self, value):
        return both.__call__(self, value) and get_spec_problems(value) == []

# ----------------------------------------------------------------------------
