'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/lib/design_patterns.py $"
__revision__ = "$Id: design_patterns.py 969 2008-03-18 18:39:49Z mario $"


class Singleton(object): 
    """ 
    A  Singleton class.
    
    Any subclass that does not override __new__ has exactly one instance.
    If the subclass defines an __init__ method, the subclass must ensure
    its __init__ is safe when called repeatedly on the same unique instance 
    (at each creation request). 
    c.f. Py Nutshell, "New Style Classes". 
    """ 
    _singletons = {}
    def __new__(cls, *args, **kw):
        if not cls._singletons.has_key(cls):
            cls._singletons[cls] = object.__new__(cls)
        return cls._singletons[cls]


class bunch(dict):
    '''
    A dictionary-bunch of values, with convenient dotted access.
    Limitation: keys must be valid object attribute names.
    Inspiration: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52308
    '''
    def __init__(self, **kwds):
        dict.__init__(self, kwds)
        self.__dict__ = self

    def __getstate__(self):
        return self

    def __setstate__(self, state):
        self.update(state)
        self.__dict__ = self

    def update_from_object(self, obj, no_underscored=True):
        ''' (obj:object) -> None
        Sets each key in dir(obj) as key=getattr(object, key) in self.
        If no_underscored, then exclude "private" (leading "__") and 
        "protected" (leading "_") attributes.
        '''
        for name in dir(obj):
            if not (no_underscored and name.startswith('_')):
                self[name] = getattr(obj, name)


class Null(Singleton):
    ''' A (singleton) class for implementing Null objects.

    This class ignores all parameters passed when constructing or 
    calling instances and traps all attribute and method requests. 
    Instances of it always (and reliably) do 'nothing'.

    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/68205
    '''

    # object constructing
        
    def __init__(self, *args, **kwargs):
        "Ignore parameters."
        return None

    # object calling

    def __call__(self, *args, **kwargs):
        "Ignore method calls."
        return self

    # attribute handling

    def __getattr__(self, mname):
        "Ignore attribute requests."
        return self

    def __setattr__(self, name, value):
        "Ignore attribute setting."
        return self

    def __delattr__(self, name):
        "Ignore deleting attributes."
        return self

    # misc.

    def __repr__(self):
        "Return a string representation."
        return '<%s>'%(self.__class__.__name__)

    def __str__(self):
        "Convert to a string and return it."
        return ''

