'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/pub/common.py $"
__revision__ = "$Id: common.py 975 2008-03-19 10:44:16Z mario $"

'''
Convenience functions for common operations that go through the publisher. 
Gizmo applications should always access these functions via this module, 
i.e. via gz.pub.common and not through qp.pub.common. 
'''
import copy
from qp.pub.common import get_publisher, site_now
from gz.fill.dhi import Dhi


''' 
Utilities based on QComps. 

A qcomp is a "Qualified Component", implemented as a 2-tuple.

    qcomps: [qcomp]
    qcomp: tuple(directory:directory, component:str)

- A list of traversed qcomps per Hit (set-up in Directory._q_traverse())
  is cached at get_info()['qcomps']. Retrieved with get_qcomps().
- The one qcomp, if any, that has been transferred to after a call to
  lock_responding_now(). Retrieved with get_respond_now_qcomp().
  
'''
    
def get_dir():
    ''' () -> directory
    Return the directory instance that is handling this response. 
    This is: 
    EITHER  the (currently) last traversed directory, 
        OR  the directory transferred to, handling this response, 
            after a call to Publisher.lock_responding_now()
    If you want to get the last traversed directory (whether or not 
    the request handling has been transferred to another directory) 
    use get_qcomp(n=-1) instead.
    '''
    return get_effective_qcomp()[0] 

def get_dhi():
    ''' () -> dhi (current hit's, copy, or new)
    Gets the current hit's dhi instance, SETTING it if necessary.
    Called explicitly by pages/gizmos that want an initialized dhi object.
    - Either get off info, 
    - Or as new copy of the one defined by current dir's eXportable,
    - Or just create a new Dhi instance. 
    '''
    info = get_publisher().get_hit().get_info()
    if info.get('dhi') is not None:
        return info['dhi']
    else:
        # Responses must always work on a new/copy of dhi        
        directory, component = get_effective_qcomp()
        x = directory.get_x_comp(component) 
        if x is None or x.get('dhi') is None:
            dhi = Dhi()
        else:
            dhi = copy.deepcopy(x.dhi)
        dhi.x = x
        # Set the new/copy of dhi on hit.info
        info['dhi'] = dhi
        return dhi

def get_effective_qcomp():
    ''' () -> (directory:Directory, component:str) or None
    Returns (dir, comp) either for diverted qcomp, 
    or for the (currently) last traversed component.
    '''
    return get_respond_now_qcomp() or get_qcomp()

def get_qcomps(): 
    ''' () -> [(directory:Directory, component:str)] '''
    return get_publisher().get_hit().get_info()['qcomps']

def get_qcomp(n=-1):
    ''' (n : int = 0) -> (directory:Directory, component:str) 
    Get traversed (directory, component): 
        n=0 returns (dir, comp) for root directory
        n=-1 returns (dir, comp) for last traversed directory.
    '''
    return get_publisher().get_hit().get_info()['qcomps'][n]
    
def get_respond_now_qcomp():
    ''' () -> (directory:Directory, component:str) or None
    Returns (dir, comp) that has been diverted to, handling this response. 
    info['respond_now_qcomp'] is only set for diverted responses. 
    '''
    return get_publisher().get_hit().get_info().get('respond_now_qcomp')

def is_transferred(to=None):
    ''' (to:Directory) -> bool 
    Is, following a lock_responding_now(): 
    - the handling of this request transferred at all (to=None) ? 
    - the handling of this request transferred to directory (to!=None) ? 
    '''
    rnqc = get_respond_now_qcomp()
    if rnqc is None:
        return False
    if to is None:
        return True
    else:
        return rnqc[0]==to

def is_last_traversed(directory, component): 
    ''' (directory:Directory, component:str) -> bool 
    Is this qcomp the (currently) last traversed qcomp? 
    '''
    return (directory, component) == get_qcomp()

def is_requested_qcomp(directory, component): 
    ''' (directory:Directory, component:str) -> bool 
    Is this qcomp the requested qcomp? 
    '''
    if get_publisher().get_hit().get_info()['done_traversal']: 
        return (directory, component) == get_qcomp()
    else:
        return False

# End Utilities based on QComps.

def script_name_path(path):
    ''' (path:str) -> str 
    Returns script_name-normalized path. 
    '''
    if path is None:
        raise ValueError, 'Path cannot be None.' 
    if not path.startswith('/'):
        if path.find('://') >= 3:
            return path
        path = '/' + path
    script_name = get_request().get_script_name()
    if script_name and not path.startswith(script_name):
        path = script_name + path
    return path

def page(title, *content, **kwargs):
    return get_publisher().skin.page(title, *content, **kwargs)

# Publisher dependent methods

def get_session(): 
    return get_publisher().get_session() 
    
def get_user(): 
    return get_publisher().get_user()

def ensure_signed_in(**kwargs): 
    return get_publisher().ensure_signed_in(**kwargs)

# Following methods are effectively identical to those in QP.

def get_hit():
    return get_publisher().get_hit()

def get_info():
    return get_publisher().get_hit().get_info()
    
def get_request():
    return get_publisher().get_hit().get_request()

def get_path(n=0):
    ''' 
    Return the path of the current request, chopping off 'n' path components 
    from the right (or left if negative), i.e. reversed indexing behaviour 
    (to that of array). 

    Note that this is identical to QP's -- but is different from 
    GizmoDir.get_path() that returns the URL traversal path for the 
    directory instance. 
    '''
    print 'WARNING: common.get_path() deprecated -> use request.get_path()'
    return get_publisher().get_hit().get_request().get_path(n=n)

def get_response():
    return get_publisher().get_hit().get_response()

def get_site():
    return get_publisher().get_site()

def redirect(location, permanent=False):
    get_publisher().redirect(location, permanent=permanent)

def not_found(body=None):
    get_publisher().not_found(body=body)

def respond(title, *content, **kwargs):
    return get_publisher().respond(title, *content, **kwargs)

# end effectively identical to QP
