'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/pub/publisher.py $"
__revision__ = "$Id: publisher.py 993 2008-03-21 17:21:23Z mario $"

from os.path import join, isabs
from gz.pub.common import get_request, script_name_path
from gz.lib import json
from gz.lib.design_patterns import Null
from gz.skin.qpy_skin import QpySkin
import qp.pub.publish
import socket


class Publisher(qp.pub.publish.Publisher):

    skin_class = QpySkin

    def __init__(self, site=None, **kw):
        qp.pub.publish.Publisher.__init__(self, site, **kw)
        if self.is_live_host():
            self.set_live_configuration()
        self.skin = self.skin_class(
                        doctype=self.get('doctype', 'XHTML_10_Transitional'))
        self.null_session = Session()
        self.null_user = User('')
        self.page = self.skin.page # for qp compatibility: publisher.respond()

    # configuration, generic

    def get(self, key, default=None):
        "Get the value of a (site's) configuration variable."
        return self.configuration.get(key, default)

    def set_live_configuration(self):
        raise NotImplementedError

    def update_configuration(self, from_dict, to_dict=None):
        if to_dict is None:
            to_dict = self.configuration
        for key in from_dict: 
            if not to_dict.has_key(key):
                to_dict[key] = from_dict[key]
            elif isinstance(to_dict[key], dict):
                self.update_configuration(from_dict[key], to_dict[key])
            else:
                to_dict[key] = from_dict[key]

    def get_url_path_for(self, basekey, filename=None):
        ''' (basekey:str, filename:str) -> str 
        Return the SiteRoot-absolute url path for basekey [and filename].
        - basekey must be a key in Publisher.get('url_paths').
        - If filename is None, return base path for basekey
          else if filename is already an absolute URL return it unchanged
          (i.e. ignoring the supplied value of basekey).
        - If filename starts with a '/', ensure that it is script_named.
        '''
        if filename is None:
            return script_name_path(self.get('url_paths')[basekey])
        elif filename.find('://') >= 3:
            # Already "Protocol-absolute" URL
            return filename 
        elif filename.startswith('/'):
            # Already a "SiteRoot-absolute" URL Path, ensure script_named.
            return script_name_path(filename)
        else: 
            # "BaseKey-relative" URL Path
            return join(script_name_path(self.get('url_paths')[basekey]), 
                        filename)

    def get_file_path_for(self, basekey, filename=None):
        ''' (basekey:str, filename:str) -> str 
        Return the absolute file path for basekey [and filename].
        - basekey must be a key in Publisher.get('file_paths').
        - If filename is already an absolute path, return it unchanged
          (ignoring the supplied value of basekey).
        - If file_paths[basekey] is already an absolute path, use it as is. 
        - If file_paths[basekey] is not absolute, then assume it is relative 
          to site.get_package_directory().
        - If filename is None, return abs path for basekey.
        '''
        if filename is not None:
            if isabs(filename):
                return filename
        basepath = self.get('file_paths')[basekey]
        if not isabs(basepath):
            basepath = join(self.site.get_package_directory(), basepath)
        if filename is None:
            return basepath
        else: 
            return join(basepath, filename)

    def has_key_path(self, primarykey, subkey): 
        ''' (primarykey:str|None, subkey:str|None) -> bool
        Return whether configuration[primarykey][subkey] is defined. 
        '''
        if not self.configuration.has_key(primarykey):
            return False
        if not self.configuration[primarykey].has_key(subkey):
            return False
        return True

    # configuration, specific values

    def get_webmaster_address(self):
        return (self.configuration.get('webmaster_address') or 
                'webmaster@%s'%(socket.getfqdn()))

    def is_email_enabled(self):
        return self.configuration.get('is_email_enabled', False)

    def display_exceptions(self):
        return self.configuration.get('display_exceptions', False)

    def secure(self):
        ''' If the scheme is not https, if not secure=False in configuration,
        then redirect so that it will be.
        '''
        if self.configuration.get('secure', True):
            qp.pub.publish.Publisher.secure(self)

    def respond_robotstxt(self):
        # http://www.robotstxt.org/wc/exclusion-admin.html
        self.txt_respond_now(self.configuration.get('robotstxt', 
                                            'User-agent: *\nDisallow: /\n'))

    # responding_now, and methods that call it

    def lock_responding_now(self):
        ''' () -> bool
        Check and set flag to prevent unwanted respond_now recursion.
        If already locked -> returns True 
        If not already locked -> locks and returns False

        This is meant for the collection of conditions that trigger a 
        non-normal respond_now, e.g. login or error pages, and the first one 
        of these that calls gets "responding_now ownership" and remains 
        responsible for proper termination of the non-normal response. 

        During each request/response cycle, the "lock_responding_now" flag 
        may be set ONLY ONCE, and once locked may NOT be unlocked. 

        To get the definitive Directory instance that is handling the 
        response (whether or not the request has been diverted) use:
        gz.pub.common.get_dir()
        '''
        info = self.get_hit().get_info()
        if info.get('lock_responding_now', False):
            return True
        else:
            info['lock_responding_now'] = True
            info['dhi'] = None
            #+ refine selection of alternative handling directory
            info['respond_now_qcomp'] = (self.get_root_directory(), '')
            return False

    def not_found(self, body=None):
        if not self.lock_responding_now():
            self.respond('Not Found', body or 'Not Found.', status=404)

    def unauthorized(self, body=None):
        ''' (body:str)
        This prepares a 401 response and responds immediately.
        '''
        if not self.lock_responding_now():
            self.respond('Unauthorized', body or 'Unauthorized.', status=401)

    def hide_exception(self):
        if not self.lock_responding_now():
            self.get_hit().get_response().set_status(500)
            self.get_hit().get_response().set_body(
                self.page('Sorry', 'Temporarily unavailable.'))

    # json callbacks

    def json_callback_fs(self, directory, field_name):
        ''' (directory:directory, field_name:str) -> repond_now() or None
        Generic dispatcher method, to initialize a field_status object, 
        and call the callback method to validate the field. 

        Incoming component: <field_name>.json
        Callback method: directory.callback_fs_<field_name>(field_status) 
        Callback response is an appropriately adjusted field_status object.

        Meant to be called from within a directory's _q_lookup, that 
        intercepts requests to components matching <field_name>.json.
        '''
        try:
            callback_fs_func = getattr(directory, 
                                        'callback_fs_%s'%(field_name))
        except AttributeError, exc:
            print exc
            return None
        field_status = dict(
                name=field_name, 
                value=get_request().get_field(field_name), 
                ok=True, 
                message='')
        self.json_respond_now(callback_fs_func(field_status))

    # $begin{json_respond_now}
    def json_respond_now(self, data, status=200):
        ''' (data:any) -> None -- sets response body with 
        json dump of a python object, and raises RespondNow
        '''
        response = self.get_hit().get_response()
        response.set_status(status)
        response.set_content_type('application/json', 'utf-8') # text/json
        response.set_body(json.dumps(data))
        self.respond_now() 
    # $end{json_respond_now}

    # txt response

    def txt_respond_now(self, txt, status=200):
        response = self.get_hit().get_response()
        response.set_status(status)
        response.set_content_type('text/plain', 'utf-8')
        response.set_body(txt)
        self.respond_now() 

    # Null User, Session. Provided to simplify code that assumes 
    # a Session/User scenario, e.g. in Directory, Skin. 
    # All methods should return or evaluate to False or None. 

    def get_session(self):
        return self.null_session

    def get_user(self):
        return self.null_user

    # deprecated

    def header(self, title, **kw):
        raise Deprecated('Use publisher.skin.page instead.')
    def footer(self, **kw):
        raise Deprecated('Use publisher.skin.page instead.')

#

class User(Null):
    pass

class Session(Null):
    pass

#

class Deprecated(Exception):
    pass

#
